SystemOrganization addCategory: #KinematicTemplates!

RectangleMorph subclass: #KinematicCanvas
	instanceVariableNames: 'virtualCursor templates drawnPath lastDrawnPoint previousRealMousePosition handMorph lastVirtualCursorOffset'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KinematicTemplates'!

!KinematicCanvas methodsFor: 'as yet unclassified' stamp: 'GermanLeiva 12/13/2012 20:42'!
addTemplate: aTemplate
	templates add: aTemplate! !

!KinematicCanvas methodsFor: 'as yet unclassified' stamp: 'carlaGriggio 12/14/2012 03:23'!
applyTemplates: realDelta
	| selectedTemplates cursorPosition temp |
	selectedTemplates := templates select: [ :aTemplate | aTemplate containsPoint: lastDrawnPoint     ].
	^selectedTemplates inject: realDelta into: [ :transformation :aTemplate | 
		aTemplate transformedDelta: transformation for: lastDrawnPoint ]
	! !

!KinematicCanvas methodsFor: 'as yet unclassified' stamp: 'carlaGriggio 12/14/2012 03:58'!
handleMouseMove: anEvent

	"After drawing on a template, the virtualCursor position is different from the activeHand position. If we move the mouse after finishing a draw, the virtualCursor should start moving from the last drawing position and NOT from the activeHand position, so we update its position with an offset.
	lastVirtualCursorOffset represents the distance between the virtualCursor and the activeHand when the last drawing was finished"
	
	(lastVirtualCursorOffset isNil) ifTrue:[ virtualCursor position: self activeHand position] ifFalse:[
	virtualCursor position: (anEvent cursorPoint translateBy: lastVirtualCursorOffset negated )].
" virtualCursor position: self activeHand position."
	super handleMouseMove: anEvent
! !

!KinematicCanvas methodsFor: 'as yet unclassified' stamp: 'GermanLeiva 12/13/2012 20:41'!
handlesMouseDown: anEvent
	^true! !

!KinematicCanvas methodsFor: 'accessing' stamp: 'GermanLeiva 12/13/2012 20:41'!
handlesMouseOver: evt
	^true! !

!KinematicCanvas methodsFor: 'as yet unclassified' stamp: 'carlaGriggio 12/14/2012 23:44'!
initialize
	| tpl1 tpl2 |
	super initialize.
	
	templates := OrderedCollection new.

	virtualCursor := KinematicCursor new.
	
	tpl1 := GridTemplate new height: 300; width: 300.
	self addMorphBack: tpl1.
	self addTemplate: tpl1. 
	
	tpl2 := RadiusTemplate  new height: 400; width: 400; left: tpl1 width.
	tpl2 color: Color red.
	self addMorphBack: tpl2.
	self addTemplate: tpl2.

	self addMorph: virtualCursor.
	
"	handMorph:=(CircleMorph new width: 15; height: 15).
	 self addMorph:  handMorph."! !

!KinematicCanvas methodsFor: 'as yet unclassified' stamp: 'carlaGriggio 12/14/2012 03:54'!
mouseDown: event
	Transcript show: '>> mouseDown: ' ; cr.
	super mouseDown: event.

	lastDrawnPoint := virtualCursor position ifNil:[event cursorPoint].
	drawnPath  := LineMorph from: lastDrawnPoint to: lastDrawnPoint color: Color black width: 2.
	self addMorph: drawnPath.! !

!KinematicCanvas methodsFor: 'as yet unclassified' stamp: 'GermanLeiva 12/13/2012 20:59'!
mouseEnter: event
	Transcript show: '>> mouseEnter: '; cr.
	super mouseEnter: event.
	self addMorph: virtualCursor.
	self currentHand showTemporaryCursor: Cursor blank.! !

!KinematicCanvas methodsFor: 'as yet unclassified' stamp: 'carlaGriggio 12/14/2012 04:13'!
mouseLeave: event
	Transcript show: '>> mouseLeave: '; cr.
	super mouseLeave: event.
	self currentHand showTemporaryCursor: nil.
	virtualCursor delete.
	drawnPath := nil.
	lastDrawnPoint := nil.
	lastVirtualCursorOffset := nil.! !

!KinematicCanvas methodsFor: 'as yet unclassified' stamp: 'carlaGriggio 12/14/2012 04:18'!
mouseMove: event
	| realDelta delta |
	
	Transcript show: '>> mouseMove: '; cr.
	previousRealMousePosition ifNil: [ previousRealMousePosition := event startPoint ].	

	realDelta := self activeHand position  -  previousRealMousePosition .
	lastVirtualCursorOffset := self activeHand position - lastDrawnPoint .
	Transcript show: 'current real: ', self activeHand position   asString; cr.
	
	delta := self applyTemplates: realDelta.
	
	drawnPath setVertices: (drawnPath vertices copyWith: lastDrawnPoint + delta).
		
	virtualCursor position: lastDrawnPoint + delta. 
		
	lastDrawnPoint := lastDrawnPoint + delta.

 	self activeHand showTemporaryCursor: Cursor down hotSpotOffset: lastVirtualCursorOffset .
"	self activeHand position: lastDrawnPoint  ."

	previousRealMousePosition := previousRealMousePosition + realDelta .
"	handMorph position: self activeHand position ."

! !

!KinematicCanvas methodsFor: 'as yet unclassified' stamp: 'carlaGriggio 12/14/2012 03:59'!
mouseUp: event
	Transcript show: '>> mouseUp: '; cr.
	previousRealMousePosition := nil.

	drawnPath := nil.! !

RectangleMorph subclass: #KinematicTemplate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KinematicTemplates'!

KinematicTemplate subclass: #CompassTemplate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KinematicTemplates'!

!CompassTemplate methodsFor: 'as yet unclassified' stamp: 'carlaGriggio 12/14/2012 01:47'!
transformedDelta: realDelta for: lastDrawnPoint
	| shiftPoint origR origTheta newR newTheta finalR finalTheta |
	
	shiftPoint := lastDrawnPoint - self center.
	
	origR := shiftPoint r.
	origTheta := shiftPoint theta.
	
	newR := (shiftPoint + realDelta) r.
	newTheta := (shiftPoint + realDelta) theta.
	
	finalR := (newR - origR) * 0.1 + origR.
  	finalTheta := (newTheta - origTheta) + origTheta.
	
	^Point 
		x: finalR * finalTheta cos - shiftPoint x
		y: finalR * finalTheta sin - shiftPoint y.! !

KinematicTemplate subclass: #GridTemplate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KinematicTemplates'!

!GridTemplate methodsFor: 'as yet unclassified' stamp: 'carlaGriggio 12/14/2012 23:08'!
transformedDelta: realDelta for: lastDrawnPoint
	^ realDelta y abs > realDelta x abs
		ifTrue: [ (realDelta x * 0.1) @ (realDelta y * 1) ]
		ifFalse: [ (realDelta x * 1) @ (realDelta y * 0.1) ]! !

!KinematicTemplate class methodsFor: 'as yet unclassified' stamp: 'carlaGriggio 12/5/2012 09:47'!
openExample
	|w tpl|
	w := SystemWindow new setLabel:'Kinematic Templates'.
	tpl := self new height: 300; width: 300; top: 30.
	w addMorph: tpl.
	w openInWorld.! !

KinematicTemplate subclass: #RadiusTemplate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KinematicTemplates'!

!RadiusTemplate methodsFor: 'as yet unclassified' stamp: 'carlaGriggio 12/14/2012 23:10'!
transformedDelta: realDelta for: lastDrawnPoint
	| shiftPoint origR origTheta newR newTheta finalR finalTheta |
	
	shiftPoint := lastDrawnPoint - self center.
	
	origR := shiftPoint r.
	origTheta := shiftPoint theta.
	
	newR := (shiftPoint + realDelta) r.
	newTheta := (shiftPoint + realDelta) theta.
	
	finalR := (newR - origR) * 1 + origR.
  	finalTheta := (newTheta - origTheta)  * 0.2+ origTheta.
	
	^Point 
		x: finalR * finalTheta cos - shiftPoint x
		y: finalR * finalTheta sin - shiftPoint y.! !

KinematicTemplate subclass: #StraightLineTemplate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KinematicTemplates'!

!StraightLineTemplate methodsFor: 'as yet unclassified' stamp: 'GermanLeiva 12/13/2012 20:16'!
transformedDelta: realDelta for: lastDrawnPoint
	^ (realDelta x * 1) @ (realDelta y * 0.1)! !

SystemWindow subclass: #KinematicWindow
	instanceVariableNames: 'virtualCursor templates drawnPath lastDrawnPoint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KinematicTemplates'!

KinematicWindow class
	instanceVariableNames: 'uniqueInstance'!

KinematicWindow class
	instanceVariableNames: 'uniqueInstance'!

!KinematicWindow class methodsFor: 'as yet unclassified' stamp: 'GermanLeiva 12/13/2012 20:56'!
openExample
	| w|
	w := self new.
	w openAsIs.

		
	! !

!KinematicWindow methodsFor: 'as yet unclassified' stamp: 'GermanLeiva 12/13/2012 20:57'!
initialize
	super initialize.
	self setLabel:'Kinematic Templates'.
	self width: 1000.
	self height: 700.
	self addMorph: (KinematicCanvas new position: 0@50; width: self width; height: self height - 40; color: Color white).! !

CircleMorph subclass: #KinematicCursor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KinematicTemplates'!

!KinematicCursor methodsFor: 'as yet unclassified' stamp: 'carlaGriggio 12/14/2012 23:44'!
initialize
	super initialize.
	self color: Color black.
	self width: 7; height: 7.! !

!KinematicCursor methodsFor: 'as yet unclassified' stamp: 'carlaGriggio 12/14/2012 23:43'!
wantsHaloFromClick
	^ false! !
